/// <reference path="../lib/babylon.2.4.d.ts" />
/// <reference path="../lib/jquery.d.ts" />
var debug = window.location.search.indexOf("debug") != -1;
function writeDebug(msg) {
    if (debug) {
        console.log("Debug: " + msg);
    }
}
function pad(num, size) {
    var s = num + "";
    while (s.length < size)
        s = "0" + s;
    return s;
}
var tempVec = new BABYLON.Vector3(0, 0, 0);
var cloneCount = 0;
var Laser = (function () {
    function Laser(scene, origin) {
        BABYLON.Effect.ShadersStore["customVertexShader"] = 'precision highp float;  attribute vec3 position; attribute vec3 normal; attribute vec2 uv;  uniform mat4 worldViewProjection; uniform float time;  varying vec3 vPosition; varying vec3 vNormal; varying vec2 vUV;  void main(void) {     vec3 v = position;     gl_Position = worldViewProjection * vec4(v, 1.0);     vPosition = position;     vNormal = normal;     vUV = uv; }';
        BABYLON.Effect.ShadersStore["customFragmentShader"] = "\n#extension GL_OES_standard_derivatives : enable\nprecision highp float;   \n        \t\n// Varying\nvarying vec3 vPosition;\nvarying vec3 vNormal;\nvarying vec2 vUV;\n\n// Refs\nuniform vec3 color;\nuniform vec3 cameraPosition;\n        \t\n \nvoid main(void) {          \n    float x = vUV.x;\n    float y = vUV.y;\n    vec2 uv = -1. + 2. * vUV;\n    float a = 1. - smoothstep(-.9, 0.9, abs(uv.x)); //*(1.-vUV.y))*1.);\n    vec3 col = vec3(1., 0, 0.);\n    gl_FragColor = vec4(col, a);\n}";
        var laserMaterial = new BABYLON.ShaderMaterial("shader", scene, {
            vertex: "custom",
            fragment: "custom"
        }, {
            needAlphaBlending: true,
            attributes: ["position", "normal", "uv"],
            uniforms: ["time", "worldViewProjection"]
        });
        this.laserCylinder = BABYLON.MeshBuilder.CreateCylinder("Laser", { height: 1, diameterTop: 0.02, diameterBottom: 0.03 }, scene);
        var matrix = BABYLON.Matrix.Translation(0, 0.5, 0);
        this.laserCylinder.setPivotMatrix(matrix);
        this.laserCylinder.parent = origin;
        this.laserCylinder.rotation.x = Math.PI / 2;
        laserMaterial.setColor3('color', new BABYLON.Color3(1, 0, 0));
        laserMaterial.setVector3('cameraPosition', BABYLON.Vector3.Zero());
        laserMaterial.setFloat('time', 0.0);
        laserMaterial.alphaMode = BABYLON.Engine.ALPHA_ADD;
        laserMaterial.alpha = 0.44444;
        laserMaterial.backFaceCulling = false;
        this.laserCylinder.material = laserMaterial;
    }
    Laser.prototype.setLength = function (number) {
        this.laserCylinder.scaling.y = number;
    };
    Laser.prototype.showBeam = function (show) {
        if (!show) {
            this.laserCylinder.visibility = 0;
        }
        else {
            this.laserCylinder.visibility = 1;
        }
    };
    return Laser;
}());
var Spider = (function () {
    function Spider(scene, cloneFrom, spawnId) {
        this.target = new BABYLON.Vector3(0, 0, 0);
        this.targetId = "";
        this.currentAnimation = "";
        this.hasTarget = false;
        this.scene = scene;
        this.mesh = cloneFrom.clone("Clone" + cloneCount, cloneFrom.parent);
        this.mesh.skeleton = cloneFrom.skeleton.clone("CloneSkel" + cloneCount, "CloneSkelId" + cloneCount);
        this.mesh.skeleton.prepare();
        cloneCount++;
        this.reset(spawnId);
    }
    Spider.prototype.reset = function (spawnId) {
        this.stopWalk = 0;
        this.hp = 3;
        this.remain = 120;
        this.targetId = spawnId;
        this.mesh.visibility = 1;
        this.speed = 0.005 + Math.random() * 0.01;
        var scale = 0.05;
        scale += Math.random() * 0.02;
        this.mesh.scaling.copyFromFloats(scale, scale, scale);
        this.beamToTarget();
        this.findNextTarget();
        this.adjustToTarget();
    };
    Spider.prototype.takeDamage = function () {
        this.stopWalk = 5;
    };
    Spider.prototype.update = function () {
        if (!this.hasTarget) {
            this.remain--;
            if (this.remain < 30 && this.mesh.visibility != 0) {
                this.mesh.visibility -= 0.03;
                if (this.mesh.visibility < 0) {
                    this.mesh.visibility = 0;
                }
            }
            return false;
        }
        if (this.stopWalk > 0) {
            this.stopWalk--;
        }
        else {
            if (this.target.x < this.mesh.position.x) {
                this.mesh.position.x -= this.speed;
            }
            if (this.target.x > this.mesh.position.x) {
                this.mesh.position.x += this.speed;
            }
            if (this.target.y < this.mesh.position.y) {
                this.mesh.position.y -= this.speed;
            }
            if (this.target.y > this.mesh.position.y) {
                this.mesh.position.y += this.speed;
            }
            if (this.target.z < this.mesh.position.z) {
                this.mesh.position.z -= this.speed;
            }
            if (this.target.z > this.mesh.position.z) {
                this.mesh.position.z += this.speed;
            }
        }
        this.target.subtractToRef(this.mesh.position, tempVec);
        if (tempVec.length() < 0.1) {
            this.findNextTarget();
            return this.adjustToTarget();
        }
        return false;
    };
    Spider.prototype.findNextTarget = function () {
        this.targetId = this.targetId.substr(0, 7) + pad((Number(this.targetId.substr(7, 3)) + 1), 3);
        this.adjustToTarget();
    };
    Spider.prototype.adjustToTarget = function () {
        var targetMesh = this.scene.getMeshByName(this.targetId);
        if (targetMesh == null) {
            this.setAnimation("Idle", 1.0);
            this.hasTarget = false;
            return true;
        }
        this.mesh.rotation.copyFromFloats(0, 0, 0);
        this.target.copyFrom(targetMesh.position);
        var area = targetMesh.getBoundingInfo().boundingBox.extendSize.x * targetMesh.scaling.x;
        this.target.x += area / 2 - Math.random() * area;
        area = targetMesh.getBoundingInfo().boundingBox.extendSize.z * targetMesh.scaling.z;
        this.target.z += area / 2 - Math.random() * area;
        this.mesh.lookAt(this.target, 0, 0, 0);
        if (Math.abs(this.target.y - this.mesh.position.y) > 0.1) {
            this.mesh.rotation.y = 0;
        }
        this.hasTarget = true;
        return false;
    };
    Spider.prototype.beamToTarget = function () {
        var targetMesh = this.scene.getMeshByName(this.targetId);
        if (targetMesh == null) {
            writeDebug("Spawn error");
        }
        this.mesh.position.copyFrom(targetMesh.position);
        var area = targetMesh.getBoundingInfo().boundingBox.extendSize.x * targetMesh.scaling.x;
        this.target.x += area / 2 - Math.random() * area;
        area = targetMesh.getBoundingInfo().boundingBox.extendSize.z * targetMesh.scaling.z;
        this.target.z += area / 2 - Math.random() * area;
        this.mesh.rotation.copyFrom(targetMesh.rotation);
        this.setAnimation("Walk", this.speed * 200);
    };
    Spider.prototype.setAnimation = function (animation, speed, repeat) {
        if (repeat === void 0) { repeat = true; }
        if (this.currentAnimation != animation) {
            this.currentAnimation = animation;
            this.mesh.skeleton.beginAnimation(animation, repeat, speed);
        }
    };
    Spider.prototype.die = function () {
        this.hasTarget = false;
        this.setAnimation("Die", 1, false);
    };
    return Spider;
}());
var Main = (function () {
    function Main(scene, canvas) {
        this.spiders = [];
        this.lightTarget = new BABYLON.Vector3(0, 2, 0);
        this.maxKills = 0;
        this.keyWasUp = true;
        this.lastPos = new BABYLON.Vector3(0, 0, 0);
        this.decals = [];
        this.scene = scene;
        this.laserSound = new BABYLON.Sound("laser", "data/laser.ogg", scene, null, { loop: true });
        this.spiderSound = new BABYLON.Sound("spider", "data/tick.ogg", scene, null, { loop: false });
        this.biteSound = new BABYLON.Sound("bite", "data/bite.ogg", scene, null, { loop: false });
        this.dieSound = new BABYLON.Sound("die", "data/die.ogg", scene, null, { loop: false });
        BABYLON.Engine.audioEngine.setGlobalVolume(0.4);
        var freeCam = new BABYLON.FreeCamera("freeCam", new BABYLON.Vector3(0, 0, 0), scene);
        if (debug) {
            freeCam.attachControl(canvas);
            freeCam.inertia = 0.4;
            freeCam.speed = 2;
        }
        freeCam.position.copyFrom(scene.activeCamera.position);
        freeCam.rotation.copyFrom(scene.activeCamera.rotation);
        scene.activeCamera = freeCam;
        // freeCam.position.copyFromFloats(6.002191011576662, 7.68229178548806, 0.4123990258512989);
        // freeCam.rotation.copyFromFloats(0.8457104473412064, -1.561836126646447, 0);
        // freeCam.position.copyFromFloats(4.597707427431548, 4.005226967613771, -2.5576361918567354);
        // freeCam.rotation.copyFromFloats(0.5638356460803715, -0.934088077134084, 0);
        freeCam.position.copyFromFloats(4.621533437166909, 3.6974588659109244, -1.801053519615937);
        freeCam.rotation.copyFromFloats(0.5853129834518362, -1.0568655194236944, 0);
        var canvas2d = new BABYLON.ScreenSpaceCanvas2D(scene, {
            id: "ScreenCanvas",
            backgroundFill: "#ffffFF09",
            backgroundRoundRadius: 10,
            x: 50,
            y: 50,
            size: new BABYLON.Size(300, 100)
        });
        this.lifesTtext = new BABYLON.Text2D("5/5", {
            parent: canvas2d,
            x: 10,
            y: 10,
            fontName: "bold 32px Helvetica, Arial, sans-serif"
        });
        this.lifesTtext.defaultFontColor = new BABYLON.Color4(0.2, 0, 0, 1);
        this.killsText = new BABYLON.Text2D("0", {
            parent: canvas2d,
            x: 10,
            y: 40 + 10,
            fontName: "bold 32px Helvetica, Arial, sans-serif"
        });
        this.killsText.defaultFontColor = new BABYLON.Color4(0.4, 0.4, 0.4, 1);
        scene.registerBeforeRender(this.beforeRender.bind(this));
        window.addEventListener("mouseup", this.mouseUp.bind(this));
        window.addEventListener("mousedown", this.mouseDown.bind(this));
        window.addEventListener("mousemove", this.mouseMove.bind(this));
        window.addEventListener("keyup", this.keyUp.bind(this));
        window.addEventListener("keydown", this.keyDown.bind(this));
        var spider = scene.getMeshByName("Spider");
        spider.visibility = 0;
        this.spiderBlueprint = spider;
        var lightPos = new BABYLON.Vector3(0, 2, 0);
        var light = new BABYLON.PointLight("searchLight", lightPos, scene);
        light.intensity = 0.9;
        this.light = light;
        var shadowGenerator = new BABYLON.ShadowGenerator(1024, light);
        shadowGenerator.usePoissonSampling = true;
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Bed"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Board"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Board2"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Table"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Chair"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Chair2"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Kitchen"));
        shadowGenerator.getShadowMap().renderList.push(scene.getMeshByName("Cubboard"));
        var room = scene.getMeshByName("Room");
        room.receiveShadows = true;
        this.shadowGen = shadowGenerator;
        this.globalLight = scene.lights[0];
        for (var _i = 0, _a = scene.meshes; _i < _a.length; _i++) {
            var mesh = _a[_i];
            if (mesh.name.indexOf("Spawn") == 0) {
                mesh.visibility = 0;
                if (debug) {
                    mesh.visibility = 0.4;
                }
            }
            if (mesh.name == "LightSelectPlane") {
                mesh.visibility = 0;
            }
        }
        this.gun = scene.getMeshByName("Gun");
        this.laser = new Laser(scene, this.gun);
        var decalMaterial = new BABYLON.StandardMaterial("decalMat", scene);
        decalMaterial.diffuseTexture = new BABYLON.Texture("data/burn.png", scene);
        decalMaterial.diffuseTexture.hasAlpha = true;
        decalMaterial.zOffset = -2;
        this.decalMat = decalMaterial;
        var particleSystem = new BABYLON.ParticleSystem("particles", 100, scene);
        //Texture of each particle
        particleSystem.particleTexture = new BABYLON.Texture("data/spark.png", scene);
        // Where the particles come from
        particleSystem.emitter = new BABYLON.Vector3(0, 0, 0);
        particleSystem.minEmitBox = new BABYLON.Vector3(0, 0, 0); // Starting all from
        particleSystem.maxEmitBox = new BABYLON.Vector3(0, 0, 0); // To...
        // Colors of all particles
        particleSystem.color1 = new BABYLON.Color4(1, 1, 0, 1.0);
        particleSystem.color2 = new BABYLON.Color4(1, 0.8, 0.8, 1.0);
        particleSystem.colorDead = new BABYLON.Color4(0.5, 0.4, 0, 0.0);
        // Size of each particle (random between...
        particleSystem.minSize = 0.06;
        particleSystem.maxSize = 0.08;
        // Life time of each particle (random between...
        particleSystem.minLifeTime = 0.2;
        particleSystem.maxLifeTime = 0.3;
        // Emission rate
        particleSystem.emitRate = 100;
        particleSystem.blendMode = BABYLON.ParticleSystem.BLENDMODE_STANDARD;
        // Set the gravity of all particles
        particleSystem.gravity = new BABYLON.Vector3(0, -50, 0);
        // Direction of each particle after it has been emitted
        particleSystem.direction1 = new BABYLON.Vector3(-1, 2, 1);
        particleSystem.direction2 = new BABYLON.Vector3(1, 3, -1);
        // Speed
        particleSystem.minEmitPower = 1;
        particleSystem.maxEmitPower = 3;
        particleSystem.updateSpeed = 0.005;
        this.sparks = particleSystem;
        this.reset();
        //(<BABYLON.StandardMaterial>this.spider.material).diffuseColor = new BABYLON.Color3(1, 0, 0);
    }
    Main.prototype.reset = function () {
        this.nextDamageTick = 0;
        this.canReset = false;
        this.light.range = 2.5;
        this.lifes = 5;
        if (this.kills > this.maxKills) {
            this.maxKills = this.kills;
        }
        this.kills = 0;
        this.rate = 200;
        this.nextSpawn = 60;
        this.globalLight.intensity = 0.4;
        for (var _i = 0, _a = this.spiders; _i < _a.length; _i++) {
            var spider = _a[_i];
            spider.remain = 0;
            spider.hasTarget = false;
        }
        while (this.decals.length > 0) {
            var dec = this.decals.shift();
            dec.dispose();
        }
    };
    Main.prototype.spawnSpider = function () {
        var spider = null;
        for (var i = 0; i < this.spiders.length; i++) {
            if (this.spiders[i].mesh.visibility == 0) {
                spider = this.spiders[i];
                break;
            }
        }
        var spawnNum = 0;
        if (Math.random() > 0.66) {
            spawnNum = 1;
        }
        if (Math.random() > 0.66) {
            spawnNum = 2;
        }
        var spawnId = "Spawn" + spawnNum + ".000";
        if (spider != null) {
            spider.reset(spawnId);
        }
        else {
            if (this.spiders.length > 20) {
                return; //20 is enough spiders for one night
            }
            spider = new Spider(this.scene, this.spiderBlueprint, spawnId);
            this.spiders.push(spider);
            this.shadowGen.getShadowMap().renderList.push(spider.mesh);
        }
        if (!this.spiderSound.isPlaying && this.lifes > 0) {
            this.spiderSound.play();
        }
    };
    Main.prototype.keyUp = function () {
        this.keyWasUp = true;
    };
    Main.prototype.keyDown = function (e) {
        if (!this.keyWasUp) {
            return;
        }
        this.keyWasUp = false;
        if (e.keyCode == 83 || e.keyCode == "S") {
            if (BABYLON.Engine.audioEngine.getGlobalVolume() == 0) {
                BABYLON.Engine.audioEngine.setGlobalVolume(0.4);
            }
            else {
                BABYLON.Engine.audioEngine.setGlobalVolume(0);
            }
        }
    };
    Main.prototype.mouseDown = function () {
        if (this.lifes <= 0) {
            if (this.canReset) {
                this.reset();
            }
            return;
        }
        if (this.lifes > 0) {
            this.laserActive = true;
        }
    };
    Main.prototype.findSpiderByMesh = function (mesh) {
        for (var _i = 0, _a = this.spiders; _i < _a.length; _i++) {
            var spider = _a[_i];
            if (spider.mesh == mesh)
                return spider;
        }
        return null;
    };
    Main.prototype.mouseMove = function () {
        var pickResult = this.scene.pick(this.scene.pointerX, this.scene.pointerY, function (m) {
            return m.name == "LightSelectPlane";
        });
        if (pickResult != null && pickResult.hit) {
            this.lightTarget.copyFrom(pickResult.pickedPoint);
            this.lightTarget.y = 2;
        }
        pickResult = this.scene.pick(this.scene.pointerX, this.scene.pointerY, function (m) {
            return m.visibility > 0;
        });
        if (pickResult != null && pickResult.hit) {
            this.gun.lookAt(pickResult.pickedPoint, 0, Math.PI, 0);
            pickResult.pickedPoint.subtractToRef(this.gun.position, tempVec);
            this.laser.setLength(tempVec.length());
        }
    };
    Main.prototype.mouseUp = function () {
        this.laserActive = false;
        if (debug) {
            console.log("p  " + this.scene.activeCamera.position.x + ", " + this.scene.activeCamera.position.y + ", " + this.scene.activeCamera.position.z);
            console.log("r  " + this.scene.activeCamera.rotation.x + ", " + this.scene.activeCamera.rotation.y + ", " + this.scene.activeCamera.rotation.z);
        }
    };
    Main.prototype.beforeRender = function () {
        if (this.nextDamageTick > 0) {
            this.nextDamageTick--;
        }
        if (!this.laserActive) {
            if (this.laserSound.isPlaying) {
                this.laserSound.stop();
            }
            if (this.sparks.isStarted()) {
                this.sparks.stop();
            }
        }
        if (this.laserActive) {
            if (!this.sparks.isStarted()) {
                this.sparks.start();
            }
            if (!this.laserSound.isPlaying) {
                this.laserSound.play();
            }
            this.kills -= 0.1;
            var pickResult = this.scene.pick(this.scene.pointerX, this.scene.pointerY, function (m) {
                return m.visibility > 0 && m.name != "Laser";
            });
            if (pickResult.hit) {
                this.sparks.emitter.copyFrom(pickResult.pickedPoint);
            }
            if (pickResult.hit && !this.lastPos.equals(pickResult.pickedPoint) && pickResult.pickedMesh.name.indexOf("Clone") == -1 && pickResult.pickedMesh.name.indexOf("decal") == -1) {
                if (pickResult.pickedMesh.name == "Face") {
                    this.lifes = 0;
                }
                this.lastPos.copyFrom(pickResult.pickedPoint);
                var decalSize = new BABYLON.Vector3(0.08, 0.08, 0.08);
                var newDecal = BABYLON.Mesh.CreateDecal("decal", pickResult.pickedMesh, pickResult.pickedPoint, pickResult.getNormal(true), decalSize, 0);
                newDecal.material = this.decalMat;
                this.decals.push(newDecal);
            }
            if (this.decals.length > 100) {
                var oldDecal = this.decals.shift();
                oldDecal.dispose();
            }
        }
        this.laser.showBeam(this.laserActive || this.nextDamageTick > 0);
        if (this.laserActive) {
            if (this.nextDamageTick == 0) {
                this.nextDamageTick = 5; //damage every 5 frames
                var pickResult = this.scene.pick(this.scene.pointerX, this.scene.pointerY, function (m) {
                    return m.name.indexOf("Clone") == 0;
                });
                if (pickResult != null) {
                    var spider = this.findSpiderByMesh(pickResult.pickedMesh);
                    if (spider != null && spider.remain > 0) {
                        spider.hp--;
                        if (spider.hp > 0) {
                            spider.takeDamage();
                        }
                        else {
                            if (spider.hp == 0) {
                                spider.die();
                                this.dieSound.play();
                                this.kills += 10;
                                if (this.rate > 20) {
                                    this.rate -= 3;
                                    if (this.kills > 100) {
                                        this.rate -= 2;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        if (this.maxKills == 0) {
            this.killsText.text = "" + this.kills.toFixed(2);
        }
        else {
            this.killsText.text = "" + this.kills.toFixed(2) + " (" + this.maxKills.toFixed(2) + ")";
        }
        this.lifesTtext.text = this.lifes + "/5";
        if (this.lifes <= 0) {
            this.laserActive = false;
            if (this.globalLight.intensity > 0) {
                this.globalLight.intensity -= 0.01;
            }
            if (this.light.range > 0.01) {
                this.light.range -= 0.01;
            }
            if (this.light.range <= 1.5) {
                if (!this.canReset) {
                    this.logscore();
                }
                this.canReset = true;
            }
        }
        this.nextSpawn--;
        if (this.nextSpawn <= 0) {
            this.spawnSpider();
            this.nextSpawn = this.rate / 2 + Math.random() * this.rate / 2;
        }
        if (this.globalLight.intensity > 0) {
            this.globalLight.intensity -= 0.0001;
        }
        for (var _i = 0, _a = this.spiders; _i < _a.length; _i++) {
            var spider = _a[_i];
            if (spider.update()) {
                if (this.lifes > 0) {
                    this.lifes--;
                    this.biteSound.play();
                }
            }
        }
        this.light.position.copyFrom(BABYLON.Vector3.Lerp(this.light.position, this.lightTarget, 0.1));
        this.light.position.y = 2;
    };
    Main.prototype.logscore = function () {
        if (this.kills == 0)
            return;
    };
    return Main;
}());
//# sourceMappingURL=main.js.map