/// <reference path="../lib/babylon.2.4.d.ts" />
/// <reference path="main.ts" />

class CustomLoadinScreen implements BABYLON.ILoadingScreen {
    loadingUIBackgroundColor: string;
    loadingUIText: string;
    animationTimeout;

    displayLoadingUI() {
        let htmlElement = document.getElementById("jploading");
        htmlElement.style.opacity = "1";
        htmlElement.style.display = "block";
        this.animationTimeout = window.setTimeout(this.updateAnimation.bind(this), 200)
    }

    updateAnimation() {
        let htmlElement = document.getElementById("jploadingdots");
        if (htmlElement.innerHTML == "...") {
            htmlElement.innerHTML = "";
        } else if (htmlElement.innerHTML == "") {
            htmlElement.innerHTML = ".";
        } else if (htmlElement.innerHTML == ".") {
            htmlElement.innerHTML = "..";
        } else {
            htmlElement.innerHTML = "...";
        }
        this.animationTimeout = window.setTimeout(this.updateAnimation.bind(this), 200)
    }

    hideLoadingUI() {
        window.clearTimeout(this.animationTimeout);
        let htmlElement = document.getElementById("jploading");
        htmlElement.style.opacity = "0";
        htmlElement.addEventListener("transitioned", function () {
            this.style.display = "none"
        }.bind(htmlElement));
    }

}

window.addEventListener('DOMContentLoaded', () => {
    const canvas = <HTMLCanvasElement>document.getElementById('renderCanvas');
    const engine = new BABYLON.Engine(canvas, true);
    engine.enableOfflineSupport = false;
    engine.loadingScreen = new CustomLoadinScreen();
    engine.displayLoadingUI();

    BABYLON.SceneLoader.ShowLoadingScreen = false;

    BABYLON.SceneLoader.Load("data/", "room.babylon", engine, function (scene) {
        scene.executeWhenReady(function () {
            //BABYLON.SceneLoader.ImportMesh
            new Main(scene, canvas);

            engine.hideLoadingUI();

            engine.runRenderLoop(function () {
                scene.render();
            });
        });
    });


    function resize() {
        let height = window.innerHeight;
        let width = window.innerWidth;

        let newHeight = width / (16/9);
        if (newHeight <= height) {
            engine.getRenderingCanvas().style.width = "100%";
            engine.getRenderingCanvas().style.height = newHeight + "px";
        } else {
            engine.getRenderingCanvas().style.width = height / (9/16) + "px";
            engine.getRenderingCanvas().style.height = "100%";
        }
        engine.resize();
    }

    window.addEventListener('resize', () => {
        resize();
    });

    resize();

});